==> javie-0.3.0-src.tar.gz: javie-0.3.0-src/ch.kuramo.javie.core/src/main/java/ch/kuramo/javie/core/internal/CompositionItemImpl.java <==
/*
 * Copyright (c) 2009 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core.internal;

import net.arnx.jsonic.JSONHint;
import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.api.VideoBounds;
import ch.kuramo.javie.core.AbstractItem;
import ch.kuramo.javie.core.AudioBuffer;
import ch.kuramo.javie.core.Composition;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.MediaInput;
import ch.kuramo.javie.core.Project;
import ch.kuramo.javie.core.ProjectDecodeException;
import ch.kuramo.javie.core.VideoBuffer;
import ch.kuramo.javie.core.WrappedOperation;
import ch.kuramo.javie.core.annotations.ProjectElement;
import ch.kuramo.javie.core.services.AudioRenderContext;
import ch.kuramo.javie.core.services.VideoRenderContext;

import com.google.inject.Inject;

@ProjectElement("composition")
public class CompositionItemImpl extends AbstractItem implements CompositionItem {

	private String _compositionId;

	private Composition _composition;

	private final CompositionInput _input = new CompositionInput();

	@Inject
	private VideoRenderContext _vrContext;

	@Inject
	private AudioRenderContext _arContext;


	public CompositionItemImpl() {
		super();
	}

	public void initialize(Composition composition) {
		super.initialize();
		_compositionId = composition.getId();
		_composition = composition;
	}

	public void dispose() {
		_composition = null;
	}

	public String getCompositionId() {
		return _compositionId;
	}

	public void setCompositionId(String compositionId) {
		_compositionId = compositionId;
	}

	@JSONHint(ignore=true)
	public Composition getComposition() {
		return _composition;
	}

	@Override
	public void afterDecode(Project p) throws ProjectDecodeException {
		super.afterDecode(p);

		_composition = p.getComposition(_compositionId);
		if (_composition == null) {
			throw new ProjectDecodeException(
					"no such Composition found: id=" + _compositionId);
		}
	}

	@JSONHint(ignore=true)
	public MediaInput getMediaInput() {
		return _input;
	}

	@JSONHint(ignore=true)
	public String getKind() {
		return "コンポジション";
	}


	private class CompositionInput implements MediaInput {

		public void dispose() {
			// noting to do
		}

		public boolean isVideoAvailable() {
			// TODO コンポジションがビデオを持っている場合だけtrueを返すようにする。
			return true;
		}

		public boolean isAudioAvailable() {
			// TODO コンポジションがオーディオを持っている場合だけtrueを返すようにする。
			return true;
		}

		public Time getDuration() {
			return _composition.getDuration();
		}

		public Time getVideoFrameDuration() {
			return _composition.getFrameDuration();
		}

		public VideoBounds getVideoFrameBounds() {
			Size2i size = _composition.getSize();
			return new VideoBounds(size.width, size.height);  // TODO 毎回newするのは無駄
		}

		public VideoBuffer getVideoFrameImage() {
			return _vrContext.saveAndExecute(new WrappedOperation<VideoBuffer>() {
				public VideoBuffer execute() {
					return _composition.renderVideoFrame();
				}
			});
		}

		public AudioBuffer getAudioChunk() {
			return _arContext.saveAndExecute(new WrappedOperation<AudioBuffer>() {
				public AudioBuffer execute() {
					return _composition.renderAudioChunk();
				}
			});
		}

	}

}

==> javie-0.3.1-src.tar.gz: javie-0.3.1-src/ch.kuramo.javie.core/src/main/java/ch/kuramo/javie/core/internal/CompositionItemImpl.java <==
/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core.internal;

import net.arnx.jsonic.JSONHint;
import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.api.VideoBounds;
import ch.kuramo.javie.core.AbstractItem;
import ch.kuramo.javie.core.AudioBuffer;
import ch.kuramo.javie.core.Composition;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.MediaInput;
import ch.kuramo.javie.core.Project;
import ch.kuramo.javie.core.ProjectDecodeException;
import ch.kuramo.javie.core.VideoBuffer;
import ch.kuramo.javie.core.WrappedOperation;
import ch.kuramo.javie.core.annotations.ProjectElement;
import ch.kuramo.javie.core.services.AudioRenderContext;
import ch.kuramo.javie.core.services.VideoRenderContext;
import ch.kuramo.javie.core.services.VideoRenderSupport;

import com.google.inject.Inject;

@ProjectElement("composition")
public class CompositionItemImpl extends AbstractItem implements CompositionItem {

	private String _compositionId;

	private Composition _composition;

	private final CompositionInput _input = new CompositionInput();

	@Inject
	private VideoRenderContext _vrContext;

	@Inject
	private VideoRenderSupport _vrSupport;

	@Inject
	private AudioRenderContext _arContext;


	public CompositionItemImpl() {
		super();
	}

	public void initialize(Composition composition) {
		super.initialize();
		_compositionId = composition.getId();
		_composition = composition;
	}

	public void dispose() {
		_composition = null;
	}

	public String getCompositionId() {
		return _compositionId;
	}

	public void setCompositionId(String compositionId) {
		_compositionId = compositionId;
	}

	@JSONHint(ignore=true)
	public Composition getComposition() {
		return _composition;
	}

	@Override
	public void afterDecode(Project p) throws ProjectDecodeException {
		super.afterDecode(p);

		_composition = p.getComposition(_compositionId);
		if (_composition == null) {
			throw new ProjectDecodeException(
					"no such Composition found: id=" + _compositionId);
		}
	}

	@JSONHint(ignore=true)
	public MediaInput getMediaInput() {
		return _input;
	}

	@JSONHint(ignore=true)
	public String getKind() {
		return "コンポジション";
	}


	private class CompositionInput implements MediaInput {

		public void dispose() {
			// noting to do
		}

		public boolean isVideoAvailable() {
			// TODO コンポジションがビデオを持っている場合だけtrueを返すようにする。
			return true;
		}

		public boolean isAudioAvailable() {
			// TODO コンポジションがオーディオを持っている場合だけtrueを返すようにする。
			return true;
		}

		public Time getDuration() {
			return _composition.getDuration();
		}

		public Time getVideoFrameDuration() {
			return _composition.getFrameDuration();
		}

		public VideoBounds getVideoFrameBounds() {
			Size2i size = _composition.getSize();
			return new VideoBounds(size.width, size.height);  // TODO 毎回newするのは無駄
		}

		public VideoBuffer getVideoFrameImage() {
			VideoBuffer vb = _vrContext.saveAndExecute(new WrappedOperation<VideoBuffer>() {
				public VideoBuffer execute() {
					return _composition.renderVideoFrame();
				}
			});
			_vrSupport.premultiply(vb);
			return vb;
		}

		public AudioBuffer getAudioChunk() {
			return _arContext.saveAndExecute(new WrappedOperation<AudioBuffer>() {
				public AudioBuffer execute() {
					return _composition.renderAudioChunk();
				}
			});
		}

	}

}
